/* -*- mode: c++; c-basic-offset: 3; -*- */
#include "woutput.hh"
#include "scandir.hh"

#include <sys/stat.h>
#include <sstream>

using namespace wpipe;
using namespace std;

//======================================  Output type constructor
wouttype::wouttype(void) {
}

//======================================  Output type destructor
wouttype::~wouttype(void) {
}

//======================================  Output type constructor
wouttype::wouttype(const str_vect& channels, const std::string& time, 
		   const std::string& dir, const std::string& fmt, 
		   const std::string& type)
   : out_dir(dir), format(fmt), trig_type(type)
{
   //------------------------------------  global file (e.g. livetime.txt)
   if (channels.size() == 1 && channels[0] == "*") {
      ostringstream oss;
      oss << out_dir  << "/" << trig_type << "-" << time << "." << format;
      channelNames.push_back(channels[0]);
      _paths.push_back(oss.str());
   }

   //------------------------------------  Trigger files...
   else {
      addChannels(channels, time);
   }
}

void
wouttype::addChannels(const str_vect& channels, const string& time) {
   size_t N = channels.size();
   if (N == 1 && channelNames[0] == "*") {
      ostringstream oss;
      oss << out_dir  << "/" << trig_type << "-" << time << "." << format;
      _paths[0] = oss.str();
   }
   else {
      for (size_t i=0; i<N; i++) {
	 string chan = channels[i];
	 channelNames.push_back(chan);
	 _paths.push_back(gen_path(out_dir, chan, time));
      }
   }
}

//======================================  Display output type
std::ostream&
wouttype::display(std::ostream& out) const {
   out << "Output directory:     " << out_dir << endl;
   out << "Format:               " << format << endl;
   out << "Trigger types:        " << trig_type << endl;
   out << "Channel List:         " << endl;
   for (size_t i=0; i<channelNames.size(); ++i) {
      out << channelNames[i] << "   " << _paths[i] << endl;
   }
   return out;
}

//======================================  Find channel number
size_t 
wouttype::find(const std::string& chan) const {
   size_t N = channelNames.size();
   if (N == 1 && channelNames[0] == "*") return 0;

   for (size_t i=0; i<N; i++) {
      if (chan == channelNames[i]) return i;
   }
   return N;
}

//======================================  Generate a path string.
std::string
wouttype::gen_path(const std::string& dir, const std::string& chan, 
		   const std::string& time) const {
   ostringstream oss;
   oss << dir  << "/" << chan.substr(0, 2) << "-OMEGA_TRIGGERS_" 
       << trig_type << "-" << time << "." << format;
   return oss.str();
}

//======================================  Update all paths
void
wouttype::update(const std::string& time) {
   size_t N = channelNames.size();
   if (N == 1 && channelNames[0] == "*") {
      ostringstream oss;
      oss << out_dir  << "/" << trig_type << "-" << time << "." << format;
      _paths[0] = oss.str();
   }
   else {
      string dir = out_dir;
      dir += "/";
      dir += time.substr(0,5);
      if (!scandir::test_directory(dir)) {
	 if (mkdir(dir.c_str(), 0755)) {
	    cerr << "unable to make directory: " << dir << endl;
	 }
      }
      for (size_t i=0; i<N; i++) {
	 _paths[i] = gen_path(dir, channelNames[i], time);
      }
   }
}

//======================================  Empty woutput constructor
woutput::woutput(void) {
}

//======================================  woutput destructor
woutput::~woutput(void) {
}

//======================================  woutput constructor
woutput::woutput(const str_vect& chans, const string& time, const string& dir, 
		 const string& fmt, const str_vect& otyp) {
   addMany(chans,time,  dir, fmt, otyp);
}

//======================================  Add entries for specified types
void
woutput::addMany(const str_vect& chans, const string& time, const string& dir, 
		 const string& fmt, const str_vect& otyp) {
   if (otyp.empty()) {
      addtype(chans, time, dir, fmt, "DOWNSELECT");
      addtype(chans, time, dir, fmt, "CLUSTER");
      addtype(chans, time, dir, fmt, "COINCIDE");
   }

   else {
      size_t N=otyp.size();
      for (size_t i=0; i<N; i++) {
	 addtype(chans, time, dir, fmt, otyp[i]);
      }
   }
}

//======================================  Add a type entry
void 
woutput::addtype(const str_vect& chans, const string& time, const string& dir, 
		 const string& fmt, const string& type) {
   omap_iter itr = out_map.find(type);
   if (itr == out_map.end()) {
      out_map.insert(omap_value(type, wouttype(chans, time, dir, fmt, type)));
   } else {
      itr->second.addChannels(chans, time);
   }
}

//======================================  Display the output file map.
std::ostream&
woutput::display(std::ostream& out) const {
   out << "Display of output map" << endl;
   out << "---------------------" << endl;
   for (const_omap_iter i=out_map.begin(); i != out_map.end(); i++) {
      out << "File definitions for "<< i->first << " data type." << endl;
      i->second.display(out);
      out << endl;
   }
   return out;
}

//======================================= Update time stamps in all files
void
woutput::update(const string& time, const str_vect& types) {
  
   if (types.empty()) {
      for (omap_iter itr=out_map.begin(); itr!=out_map.end(); ++itr) {
	 itr->second.update(time);
      }
   }

   else {
      size_t N=types.size();
      for (size_t i=0; i<N; i++) {
	 string otype = types[i];
	 omap_iter itr = out_map.find(otype);
	 if (itr == out_map.end()) continue;
	 itr->second.update(time);
      }
   }
}
